# Purpose:  Launch the main window
#
# This application is not free software; you may not redistribute 
# it or modify it.
#
# Copyright (c) 2004 Brillig Systems, Inc. All rights reserved.
#

require 'fox'
require 'SessionManager'
require 'IconManager'
require 'FileManager'
require 'ExecutionManager'
require 'GuiExtensions.rb'
require 'PropertiesFrame'
require 'EditBook'
require 'SourceBrowser'
require 'ExecutionMonitor'
require 'HomeViewer'
require 'ObjectGarden'
require 'CreateProjectDialog'
require 'PreferencesDialog'

include Fox

module Mondrian
  # command capabilities
	CMD_FILE						= 0
  CMD_UNDO        		= 1
  CMD_REDO        		= 2
  CMD_CUT         		= 3
  CMD_COPY        		= 4
  CMD_PASTE       		= 5
  CMD_DELETE      		= 6
  CMD_FIND        		= 7
  CMD_FIND_NEXT    		= 8
  CMD_FIND_PREVIOUS		= 9
	CMD_REPLACE					= 10
	CMD_REPLACE_ALL			= 11
  CMD_GOTO_LINE   		= 12
  CMD_SELECT_ALL  		= 13
	CMD_BLOCK_COMMENT		= 14
	
	# GUI features expressed as capabilities
	GUI_CURSOR_POSITION	= 50
	GUI_QUICK_FIND			= 51


	#
	# Mondrian's main window
	#
	
  class MainWindow < FXMainWindow
    def initialize(app)
      icon = FXBMPIcon.new(app, nil, FXRGB(255, 255, 255), IMAGE_ALPHACOLOR)
      FXFileStream.open("images/mondrian_icon.bmp", FXStreamLoad) do |stream| 
        icon.loadPixels(stream)
      end
      icon.create

      small_icon = FXBMPIcon.new(app, nil, FXRGB(255, 255, 255), IMAGE_ALPHACOLOR)
      FXFileStream.open("images/mondrian_icon_small.bmp", FXStreamLoad) do |stream| 
        small_icon.loadPixels(stream)
      end
      small_icon.create
			
      super(app, "Mondrian Ruby IDE", icon, small_icon, DECOR_ALL, 0, 0, 800, 600)
      
      FileManager.instance.add_observer(self)
      ExecutionManager.instance.add_observer(self)
      SessionManager.instance.add_observer(self)
      
      register_filetypes
      register_icons

      #
      # basic layout
      #
      
			FXTooltip.new(app) # initialize tooltips
		
      @menubar = FXMenubar.new(self, LAYOUT_SIDE_TOP | LAYOUT_FILL_X, 0, 0, 0, 0, 3, 3, 0, 0, 0, 0)
      
      FXHorizontalSeparator.new(self, LAYOUT_FILL_X | LAYOUT_SIDE_TOP | SEPARATOR_GROOVE)
      
      @toolbar = FXToolbar.new(self, LAYOUT_SIDE_TOP | LAYOUT_FILL_X | LAYOUT_FIX_HEIGHT, 0, 0, 0, 25, 4, 4, 0, 0, 0, 0)
      
      @statusbar = FXStatusbar.new(self, LAYOUT_SIDE_BOTTOM | LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0)
			
      vSplitter = FXSplitter.new(self, LAYOUT_FILL_X | LAYOUT_FILL_Y | SPLITTER_TRACKING | SPLITTER_VERTICAL | SPLITTER_REVERSED)
      hSplitter = FXSplitter.new(vSplitter, LAYOUT_FILL_X | LAYOUT_FILL_Y | LAYOUT_FIX_HEIGHT | SPLITTER_TRACKING | SPLITTER_HORIZONTAL, 0, 0, 0, 475)
      
      @sourceBrowser = SourceBrowser.new(hSplitter, FRAME_THICK | LAYOUT_FIX_WIDTH | LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 200, 0, 0, 0, 0, 0, 0, 0)
      @sourceBrowser.connect(SEL_UNMAP,  method(:onSidebarChange))
      @sourceBrowser.connect(SEL_MAP,    method(:onSidebarChange))

      # we could add more frames to the first splitter, but adding another makes resizing more friendly
      hSplitter = FXSplitter.new(hSplitter, LAYOUT_FILL_X | LAYOUT_FILL_Y | LAYOUT_FIX_HEIGHT | SPLITTER_TRACKING | SPLITTER_HORIZONTAL | SPLITTER_REVERSED, 0, 0, 0, 475)
      
      @editFrame = FXVerticalFrame.new(hSplitter, FRAME_NONE | LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
      
      @editBook = EditBook.new(@editFrame, nil, 0, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0)

      @propFrame = PropertiesFrame.new(hSplitter, FRAME_THICK | LAYOUT_FILL_X | LAYOUT_FILL_Y | LAYOUT_FIX_WIDTH, 0, 0, 150, 0, 0, 0, 0, 0, 0, 0)
      @propFrame.connect(SEL_UNMAP,  method(:onSidebarChange))
      @propFrame.connect(SEL_MAP,    method(:onSidebarChange))
      @propFrame.handle(self, MKUINT(ID_HIDE, SEL_COMMAND), nil)

      @reportFrame = ExecutionMonitor.new(vSplitter, FRAME_THICK | LAYOUT_FILL_X | LAYOUT_FILL_Y | LAYOUT_FIX_HEIGHT, 0, 0, 0, 100, 0, 0, 0, 0, 0, 0)
      @reportFrame.connect(SEL_UNMAP,  method(:onSidebarChange))
      @reportFrame.connect(SEL_MAP,    method(:onSidebarChange))
      
      self.connect(SEL_CLOSE, method(:onQuit))
      
      # initialize variables
      
      @gui            = {}  # a registry of gui objects such as buttons and menu items
      @focused_viewer = nil # the currently active viewer in the edit book
      
      populate_menu      
      populate_toolbar
			populate_statusbar
    end
  
    
    def register_filetypes
      fileManager = FileManager.instance
      
      fileManager.register_filetype(".mpr", ProjectFile)
      fileManager.register_filetype(".rb",  RubyFile)
      fileManager.register_filetype(".rbw", RubyFile)
			
			fileManager.default_filetype = RubyFile
    end
    
    
    def register_icons
      iconManager = IconManager.instance
      
      iconManager.register_icon(GenericFile,  IconManager.load_icon(getApp, "file.png"))
      iconManager.register_icon(ProjectFile,  IconManager.load_icon(getApp, "project.png"))
      iconManager.register_icon(RubyFile,     IconManager.load_icon(getApp, "file.png"))
      iconManager.register_icon(RubyModule,   IconManager.load_icon(getApp, "module.png"))
      iconManager.register_icon(RubyClass,    IconManager.load_icon(getApp, "class.png"))
      iconManager.register_icon(RubyMethod,   IconManager.load_icon(getApp, "method.png"))
    end


		def register_default_viewers
			@editBook.register_default_viewer(ProjectFile,  ObjectGarden::ObjectGarden)
			@editBook.register_default_viewer(RubyFile, 		RubyText)
			@editBook.register_default_viewer(RubyModule,		RubyText)
			@editBook.register_default_viewer(RubyClass, 		RubyText)
			@editBook.register_default_viewer(RubyMethod, 	RubyText)
		end
		
    
    def populate_toolbar
      @gui["toolbar.New File"]  = FXButton.new(@toolbar, "\tNew File\tCreate a new file.", IconManager.load_icon(getApp, "new.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)      
      @gui["toolbar.New File"].connect(SEL_COMMAND, method(:onNewFile))
      
      @gui["toolbar.Open File"] = FXButton.new(@toolbar, "\tOpen File\tOpen an existing file.", IconManager.load_icon(getApp, "open.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)
      @gui["toolbar.Open File"].connect(SEL_COMMAND, method(:onOpenFile))
      
      @gui["toolbar.Close File"] = FXButton.new(@toolbar, "\tClose File\tClose the current file or editor tab.", IconManager.load_icon(getApp, "close.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)
      @gui["toolbar.Close File"].connect(SEL_COMMAND, method(:onCloseFile))
      
      @gui["toolbar.Save File"] = FXButton.new(@toolbar, "\tSave File\tSave the current editor tab.", IconManager.load_icon(getApp, "save.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)
      @gui["toolbar.Save File"].connect(SEL_COMMAND, method(:onSave))

      FXVerticalSeparator.new(@toolbar, LAYOUT_FIX_HEIGHT | LAYOUT_SIDE_LEFT | SEPARATOR_GROOVE, 0, 0, 0, 25, 6, 6)
			
      @gui["toolbar.Undo"] = FXButton.new(@toolbar, "\tUndo\tUndo the last operation.", IconManager.load_icon(getApp, "undo.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)
      @gui["toolbar.Undo"].connect(SEL_COMMAND, method(:onUndo))
      
      @gui["toolbar.Redo"] = FXButton.new(@toolbar, "\tRedo\tRedo the last undone operation.", IconManager.load_icon(getApp, "redo.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)
      @gui["toolbar.Redo"].connect(SEL_COMMAND, method(:onRedo))
			
      FXVerticalSeparator.new(@toolbar, LAYOUT_FIX_HEIGHT | LAYOUT_SIDE_LEFT | SEPARATOR_GROOVE, 0, 0, 0, 25, 6, 6)
			
      @gui["toolbar.Cut"] = FXButton.new(@toolbar, "\tCut\tCut the current selection.", IconManager.load_icon(getApp, "cut.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)
      @gui["toolbar.Cut"].connect(SEL_COMMAND, method(:onCut))
			
      @gui["toolbar.Copy"] = FXButton.new(@toolbar, "\tCopy\tCopy the current selection to the clipboard.", IconManager.load_icon(getApp, "copy.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)
      @gui["toolbar.Copy"].connect(SEL_COMMAND, method(:onCopy))
			
      @gui["toolbar.Paste"] = FXButton.new(@toolbar, "\tCopy\tPaste the contents of the clipboard.", IconManager.load_icon(getApp, "paste.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)
      @gui["toolbar.Paste"].connect(SEL_COMMAND, method(:onPaste))
			
      FXVerticalSeparator.new(@toolbar, LAYOUT_FIX_HEIGHT | LAYOUT_SIDE_LEFT | SEPARATOR_GROOVE, 0, 0, 0, 25, 6, 6)
        
			frame = FXHorizontalFrame.new(@toolbar, 0, 0, 0, 0, 0, 6, 6, 0, 0) # combobox doesn't space correctly
      @gui["toolbar.Quick Find"] = FXComboBox.new(frame, 20, 10, nil, 0, COMBOBOX_NO_REPLACE | FRAME_LINE)
			@gui["toolbar.Quick Find"].connect(SEL_COMMAND, method(:onQuickFind))
			
      @gui["toolbar.Find..."] = FXButton.new(@toolbar, "\tFind\tSearch the current editor tab.", IconManager.load_icon(getApp, "find.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)
      @gui["toolbar.Find..."].connect(SEL_COMMAND, method(:onFind))
			
      FXVerticalSeparator.new(@toolbar, LAYOUT_FIX_HEIGHT | LAYOUT_SIDE_LEFT | SEPARATOR_GROOVE, 0, 0, 0, 25, 6, 6)

      @gui["toolbar.Run File"] = FXButton.new(@toolbar, "\tRun File\tRun the active file.", IconManager.load_icon(getApp, "run.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)
      @gui["toolbar.Run File"].connect(SEL_COMMAND, method(:onRunFile))
			
      @gui["toolbar.Run Project"] = FXButton.new(@toolbar, "\tRun Project\tRun the active project.", IconManager.load_icon(getApp, "run_project.png"), nil, 0, BUTTON_TOOLBAR | BUTTON_NORMAL)
      @gui["toolbar.Run Project"].connect(SEL_COMMAND, method(:onRunProject))
    end
		
		
		def populate_statusbar
			frame = FXHorizontalFrame.new(@statusbar, FRAME_SUNKEN, 0, 0, 0, 0, 0, 0, 0, 0)
			
			@gui["statusbar.Line Label"] 		= FXLabel.new(frame, "Line:")
			@gui["statusbar.Line"] 					= FXLabel.new(frame, "")
			@gui["statusbar.Column Label"] 	= FXLabel.new(frame, "Column:")
			@gui["statusbar.Column"] 				= FXLabel.new(frame, "")
			
			FXDragCorner.new(frame)
		end
    
    
    def populate_menu
      # file menu
      
      pane = FXMenuPane.new(self)
      
      @gui["menu.File/New File"]        = FXMenuCommand.new(pane, "&New File\tCtl-N\tCreate a new file.", IconManager.load_icon(getApp, "new.png"))
      @gui["menu.File/New File"].connect(SEL_COMMAND, method(:onNewFile))
      @gui["menu.File/Open File..."]    = FXMenuCommand.new(pane, "&Open File...\tCtl-O\tOpen an existing file.", IconManager.load_icon(getApp, "open.png"))
      @gui["menu.File/Open File..."].connect(SEL_COMMAND, method(:onOpenFile))
      @gui["menu.File/Save File"]       = FXMenuCommand.new(pane, "&Save File\tCtl-S\tSave the current editor tab.", IconManager.load_icon(getApp, "save.png"))
      @gui["menu.File/Save File"].connect(SEL_COMMAND, method(:onSave))
      @gui["menu.File/Save File As..."] = FXMenuCommand.new(pane, "Save File &As...\t\tSave and name the current file.")
      @gui["menu.File/Save File As..."].connect(SEL_COMMAND, method(:onSaveAs))
      @gui["menu.File/Close File"] = FXMenuCommand.new(pane, "&Close File\tCtl-W\tClose the current file or editor tab.", IconManager.load_icon(getApp, "close.png"))
      @gui["menu.File/Close File"].connect(SEL_COMMAND, method(:onCloseFile))
      
      FXMenuSeparator.new(pane)
    
      @gui["menu.File/New Project..."]  = FXMenuCommand.new(pane, "New Project...\t\tCreate a new project.")
      @gui["menu.File/New Project..."].connect(SEL_COMMAND, method(:onNewProject))
      @gui["menu.File/Open Project..."] = FXMenuCommand.new(pane, "Open &Project...\t\tOpen an existing project.")
      @gui["menu.File/Open Project..."].connect(SEL_COMMAND, method(:onOpenProject))
      @gui["menu.File/Close Project"]   = FXMenuCommand.new(pane, "Close Project\t\tClose the active project.")
			@gui["menu.File/Close Project"].connect(SEL_COMMAND, method(:onCloseProject))
      
      FXMenuSeparator.new(pane)
      
      @gui["menu.File/Save All"] = FXMenuCommand.new(pane, "Save All\t\tSave all open files.")
			@gui["menu.File/Save All"].connect(SEL_COMMAND, method(:onSaveAll))
      
      FXMenuSeparator.new(pane)
      
      @gui["menu.File/Exit"] = FXMenuCommand.new(pane, "E&xit\t\tClose all files and exit the application.", nil, getApp(), FXApp::ID_QUIT, 0)
      
      @gui["menu.File"] = FXMenuTitle.new(@menubar, "&File", nil, pane)
      
      # edit menu
      
      pane = FXMenuPane.new(self)
      
      @gui["menu.Edit/Undo"] = FXMenuCommand.new(pane, "Undo\tCtl-Z\tUndo the last operation.", IconManager.load_icon(getApp, "undo.png"))
      @gui["menu.Edit/Undo"].connect(SEL_COMMAND, method(:onUndo))
      @gui["menu.Edit/Redo"] = FXMenuCommand.new(pane, "Redo\tCtl-Y\tRedo the last undone operation.", IconManager.load_icon(getApp, "redo.png"))
      @gui["menu.Edit/Redo"].connect(SEL_COMMAND, method(:onRedo))
      
      FXMenuSeparator.new(pane)
      
      @gui["menu.Edit/Cut"]     = FXMenuCommand.new(pane, "Cut\tCtl-X\tCut the selection to the clipboard.", IconManager.load_icon(getApp, "cut.png"))
      @gui["menu.Edit/Cut"].connect(SEL_COMMAND, method(:onCut))
      @gui["menu.Edit/Copy"]    = FXMenuCommand.new(pane, "Copy\tCtl-C\tCopy the selection to the clipboard.", IconManager.load_icon(getApp, "copy.png"))
      @gui["menu.Edit/Copy"].connect(SEL_COMMAND, method(:onCopy))
      @gui["menu.Edit/Paste"]   = FXMenuCommand.new(pane, "Paste\tCtl-V\tPaste the contents of the clipboard.", IconManager.load_icon(getApp, "paste.png"))
      @gui["menu.Edit/Paste"].connect(SEL_COMMAND, method(:onPaste))
      @gui["menu.Edit/Delete"]  = FXMenuCommand.new(pane, "Delete\tDel\tDelete the current selection.")
      @gui["menu.Edit/Delete"].connect(SEL_COMMAND, method(:onDelete))
      
      FXMenuSeparator.new(pane)
      
      @gui["menu.Edit/Select All"] = FXMenuCommand.new(pane, "Select All\tCtl-A\tSelect all objects in the current editor tab.")
      @gui["menu.Edit/Select All"].connect(SEL_COMMAND, method(:onSelectAll))
     
      FXMenuSeparator.new(pane)
      
      @gui["menu.Edit/Find..."]       = FXMenuCommand.new(pane, "Find...\tCtl-F\tSearch the current editor tab.", IconManager.load_icon(getApp, "find.png"))
			@gui["menu.Edit/Find..."].connect(SEL_COMMAND, method(:onFind))
      @gui["menu.Edit/Find Next"]     = FXMenuCommand.new(pane, "Find Next\tF3\tSearch again.")
			@gui["menu.Edit/Find Next"].connect(SEL_COMMAND, method(:onFindNext))
      @gui["menu.Edit/Find Previous"] = FXMenuCommand.new(pane, "Find Previous\tShift-F3\tSearch upwards.")
			@gui["menu.Edit/Find Previous"].connect(SEL_COMMAND, method(:onFindPrevious))
      @gui["menu.Edit/Replace..."]    = FXMenuCommand.new(pane, "Replace...\tCtl-H\tSearch and replace.")
      @gui["menu.Edit/Replace..."].connect(SEL_COMMAND, method(:onReplace))
      @gui["menu.Edit/Goto Line..."]  = FXMenuCommand.new(pane, "Goto Line...\tCtl-G\tGoto a specific line number.")
			@gui["menu.Edit/Goto Line..."].connect(SEL_COMMAND, method(:onGotoLine))
      
      FXMenuSeparator.new(pane)
			
      @gui["menu.Edit/Block Comment or Uncomment"] = FXMenuCommand.new(pane, "Block Comment or Uncomment\tCtl-Q\tComment or uncomment a range of text.")
      @gui["menu.Edit/Block Comment or Uncomment"].connect(SEL_COMMAND, method(:onBlockComment))
			
      FXMenuSeparator.new(pane)
			
      @gui["menu.Edit/Preferences..."] = FXMenuCommand.new(pane, "Preferences...\t\tEdit program preferences.")
      @gui["menu.Edit/Preferences..."].connect(SEL_COMMAND, method(:onEditPreferences))
      
			@gui["menu.Edit"] = FXMenuTitle.new(@menubar, "&Edit", nil, pane)
      
      # project menu
      
      pane = FXMenuPane.new(self)
      
      @gui["menu.Project/Add File(s)..."] = FXMenuCommand.new(pane, "Add File(s)...\t\tAdd files to the project.")
      @gui["menu.Project/Add File(s)..."].connect(SEL_COMMAND, method(:onAddFiles))
      @gui["menu.Project/Add Current File"] = FXMenuCommand.new(pane, "Add Current File\t\tAdd the current file to the active project.")
      @gui["menu.Project/Add Current File"].connect(SEL_COMMAND, method(:onAddCurrentFile))
      @gui["menu.Project/Remove Current File"] = FXMenuCommand.new(pane, "Remove Current File\t\tRemove the current file from its project.")
      @gui["menu.Project/Remove Current File"].connect(SEL_COMMAND, method(:onRemoveCurrentFile))

      FXMenuSeparator.new(pane)

      @gui["menu.Project/Set Current File As Project Main"] = FXMenuCommand.new(pane, "Set Current File As Project Main\t\tMake the current file the main project file.")
      @gui["menu.Project/Set Current File As Project Main"].connect(SEL_COMMAND, method(:onSetCurrentFileAsProjectMain))
			@gui["menu.Project/Project Settings..."] = FXMenuCommand.new(pane, "Project Settings...\t\tView the project settings dialog.")
      @gui["menu.Project/Project Settings..."].connect(SEL_COMMAND, method(:onProjectSettings))
      
      FXMenuSeparator.new(pane)
      
      @gui["menu.Project/Set Active Project"] = FXMenuCascade.new(pane, "Set Active Project\t\tChange the active project.", nil, FXMenuPane.new(self))
      
      @gui["menu.Project"] = FXMenuTitle.new(@menubar, "&Project", nil, pane)
      
      # run menu
      
      pane = FXMenuPane.new(self)
      
      @gui["menu.Run/Run File"]    = FXMenuCommand.new(pane, "Run File\tF9\tRun the active file.", IconManager.load_icon(getApp, "run.png"))
      @gui["menu.Run/Run File"].connect(SEL_COMMAND, method(:onRunFile))
      @gui["menu.Run/Run Project"] = FXMenuCommand.new(pane, "Run Project\tF5\tRun the active project.", IconManager.load_icon(getApp, "run_project.png"))
      @gui["menu.Run/Run Project"].connect(SEL_COMMAND, method(:onRunProject))
#      @gui["menu.Run/Stop Execution"] = FXMenuCommand.new(pane, "Stop Execution\tCtrl-Break\tStop the currently running file.")
#      @gui["menu.Run/Stop Execution"].connect(SEL_COMMAND, method(:onStopExecution))
#			@gui["menu.Run/Stop Execution"].disable
      
      @gui["menu.Run"] = FXMenuTitle.new(@menubar, "&Run", nil, pane)
      
      # tools menu
      
      pane = FXMenuPane.new(self)
      
      @gui["menu.Tools/Edit Environment..."] = FXMenuCommand.new(pane, "Edit Environment...\t\tEdit the file's execution environment.")
			@gui["menu.Tools/Edit Environment..."].connect(SEL_COMMAND, method(:onEditEnvironment))
      @gui["menu.Tools/Edit Parameters..."] = FXMenuCommand.new(pane, "Edit Parameters...\t\tEdit the file's command-line arguments.")
			@gui["menu.Tools/Edit Parameters..."].connect(SEL_COMMAND, method(:onEditParameters))
      @gui["menu.Tools/Rename File..."] = FXMenuCommand.new(pane, "Rename File...\t\tRename the file.")
			@gui["menu.Tools/Rename File..."].connect(SEL_COMMAND, method(:onRenameFile))

      @gui["menu.Tools"] = FXMenuTitle.new(@menubar, "&Tools", nil, pane)
			
      # view menu
      
      pane = FXMenuPane.new(self)
      
      @gui["menu.View/Source Browser"]    = FXMenuCommand.new(pane, "Source Browser\tF6\tView or hide the source browser.")
      @gui["menu.View/Source Browser"].connect(SEL_COMMAND, method(:onViewSidebar))
      @gui["menu.View/Properties Frame"]  = FXMenuCommand.new(pane, "Properties Frame\tF7\tView or hide the properties frame.")
      @gui["menu.View/Properties Frame"].connect(SEL_COMMAND, method(:onViewSidebar))
			@gui["menu.View/Properties Frame"].hide
      @gui["menu.View/Output Frame"]      = FXMenuCommand.new(pane, "Output Frame\tF8\tView or hide the output frame.")
      @gui["menu.View/Output Frame"].connect(SEL_COMMAND, method(:onViewSidebar))
      @gui["menu.View/Home Page"]         = FXMenuCommand.new(pane, "Home Page\t\tView or hide the application home page.")
      @gui["menu.View/Home Page"].connect(SEL_COMMAND, method(:onViewHome))
      
      @gui["menu.View"] = FXMenuTitle.new(@menubar, "&View", nil, pane)
      
      # help menu
      
      pane = FXMenuPane.new(self)
      
      @gui["menu.Help/About Mondrian..."] = FXMenuCommand.new(pane, "About Mondrian...\t\tAbout the application.")
			@gui["menu.Help/About Mondrian..."].connect(SEL_COMMAND, method(:onAbout))
			
      @gui["menu.Help"] = FXMenuTitle.new(@menubar, "&Help", nil, pane)      
    end
    
    
    def sync_capabilities
      #
      # enable/disable global commands
      #

      if SessionManager.instance.project_count > 0
        @gui["menu.File/Close Project"].enable
        @gui["menu.Project/Add File(s)..."].enable
        @gui["menu.Project/Set Active Project"].enable
        @gui["menu.Project/Project Settings..."].enable
      else
        @gui["menu.File/Close Project"].disable
        @gui["menu.Project/Add File(s)..."].disable
        @gui["menu.Project/Set Active Project"].disable
        @gui["menu.Project/Project Settings..."].disable
      end
      
			if @focused_viewer and @focused_viewer.capable?(CMD_FILE)
				if SessionManager.instance.get_associated_project(@focused_viewer.file)
					@gui["menu.Project/Add Current File"].disable
					@gui["menu.Project/Remove Current File"].enable
					if SessionManager.instance.get_associated_project(@focused_viewer.file).is_project_key?(@focused_viewer.file)
						@gui["menu.Project/Set Current File As Project Main"].disable
					else
						@gui["menu.Project/Set Current File As Project Main"].enable
					end
				else
					if SessionManager.instance.project_count > 0
						@gui["menu.Project/Add Current File"].enable
					else
						@gui["menu.Project/Add Current File"].disable
					end
					@gui["menu.Project/Remove Current File"].disable
					@gui["menu.Project/Set Current File As Project Main"].disable
				end
			else
				@gui["menu.Project/Add Current File"].disable
				@gui["menu.Project/Remove Current File"].disable
				@gui["menu.Project/Set Current File As Project Main"].disable
			end
				
      if ExecutionManager.instance.is_running?
        @gui["menu.Run/Run File"].disable
				@gui["toolbar.Run File"].disable
        @gui["menu.Run/Run Project"].disable
				@gui["toolbar.Run Project"].disable
#        @gui["menu.Run/Stop Execution"].enable
      else
        if @focused_viewer and @focused_viewer.capable?(CMD_FILE)
          @gui["menu.Run/Run File"].enable           
          @gui["toolbar.Run File"].enable           
        else
          @gui["menu.Run/Run File"].disable
          @gui["toolbar.Run File"].disable
        end
        if SessionManager.instance.project_count > 0
          @gui["menu.Run/Run Project"].enable
          @gui["toolbar.Run Project"].enable
        else
          @gui["menu.Run/Run Project"].disable
          @gui["toolbar.Run Project"].disable
        end
#        @gui["menu.Run/Stop Execution"].disable
      end
      
      if @focused_viewer
        @gui["menu.File/Close File"].enable
        @gui["toolbar.Close File"].enable
      else
        @gui["menu.File/Close File"].disable
        @gui["toolbar.Close File"].disable
      end
      
      #
      # enable/disable based on the currently active viewer
      #
			
			if @focused_viewer and @focused_viewer.capable?(GUI_CURSOR_POSITION)
				position = @focused_viewer.get_cursor_position
				
				@gui["statusbar.Line"].text 	= position[0].to_s
				@gui["statusbar.Column"].text = position[1].to_s
			else
				@gui["statusbar.Line"].text 	= "N/A"
				@gui["statusbar.Column"].text = "N/A"
			end
      
      if @focused_viewer and @focused_viewer.capable?(CMD_SELECT_ALL)
        @gui["menu.Edit/Select All"].enable
      else
        @gui["menu.Edit/Select All"].disable
      end
      
      if @focused_viewer and @focused_viewer.capable?(CMD_BLOCK_COMMENT)
        @gui["menu.Edit/Block Comment or Uncomment"].enable
      else
        @gui["menu.Edit/Block Comment or Uncomment"].disable
      end
			
      if @focused_viewer and @focused_viewer.capable?(CMD_CUT)
        @gui["menu.Edit/Cut"].enable
        @gui["toolbar.Cut"].enable
      else
        @gui["menu.Edit/Cut"].disable
        @gui["toolbar.Cut"].disable
      end
      
      if @focused_viewer and @focused_viewer.capable?(CMD_COPY)
        @gui["menu.Edit/Copy"].enable
        @gui["toolbar.Copy"].enable
      else
        @gui["menu.Edit/Copy"].disable
        @gui["toolbar.Copy"].disable
      end
      
      if @focused_viewer and @focused_viewer.capable?(CMD_FIND)
        @gui["menu.Edit/Find..."].enable
        @gui["menu.Edit/Find Next"].enable
        @gui["menu.Edit/Find Previous"].enable
        @gui["menu.Edit/Replace..."].enable
        @gui["toolbar.Find..."].enable
      else
        @gui["menu.Edit/Find..."].disable
        @gui["menu.Edit/Find Next"].disable
        @gui["menu.Edit/Find Previous"].disable
        @gui["menu.Edit/Replace..."].disable
        @gui["toolbar.Find..."].disable
      end
      
      if @focused_viewer and @focused_viewer.capable?(CMD_GOTO_LINE)
        @gui["menu.Edit/Goto Line..."].enable
      else
        @gui["menu.Edit/Goto Line..."].disable
      end
      
      if @focused_viewer and @focused_viewer.capable?(CMD_PASTE)
        @gui["menu.Edit/Paste"].enable
        @gui["toolbar.Paste"].enable
      else
        @gui["menu.Edit/Paste"].disable
        @gui["toolbar.Paste"].disable
      end
      
      if @focused_viewer and @focused_viewer.capable?(CMD_DELETE)
        @gui["menu.Edit/Delete"].enable
      else
        @gui["menu.Edit/Delete"].disable
      end
      
      if @focused_viewer and @focused_viewer.capable?(CMD_UNDO)
        @gui["menu.Edit/Undo"].enable
        @gui["toolbar.Undo"].enable
      else
        @gui["menu.Edit/Undo"].disable
        @gui["toolbar.Undo"].disable
      end
      
      if @focused_viewer and @focused_viewer.capable?(CMD_REDO)
        @gui["menu.Edit/Redo"].enable
        @gui["toolbar.Redo"].enable
      else
        @gui["menu.Edit/Redo"].disable
        @gui["toolbar.Redo"].disable
      end
      
      if @focused_viewer and @focused_viewer.capable?(CMD_FILE)
        @gui["menu.File/Save File"].enable
        @gui["toolbar.Save File"].enable
        @gui["menu.File/Save File As..."].enable
				@gui["menu.Tools/Rename File..."].enable
				@gui["menu.Tools/Edit Parameters..."].enable
				@gui["menu.Tools/Edit Environment..."].enable
      else
        @gui["menu.File/Save File"].disable
        @gui["toolbar.Save File"].disable
        @gui["menu.File/Save File As..."].disable
				@gui["menu.Tools/Rename File..."].disable
				@gui["menu.Tools/Edit Parameters..."].disable
				@gui["menu.Tools/Edit Environment..."].disable
      end
    end
    
    
    def create
      super

      # load previous session variables and set the GUI the way it was last session      
      
      session = SessionManager.instance
      
      main_x      = session.restore("main_x")
      main_y      = session.restore("main_y")      
      main_width  = session.restore("main_width")
      main_height = session.restore("main_height")
      
      reportFrame_height  = session.restore("reportFrame_height")
      reportFrame_hidden = session.restore("reportFrame_hidden")
      @reportFrame.height = reportFrame_height if reportFrame_height != nil
      @reportFrame.handle(self, MKUINT(ID_HIDE, SEL_COMMAND), nil) if reportFrame_hidden
      
      sourceBrowser_width  = session.restore("sourceBrowser_width")
      sourceBrowser_hidden = session.restore("sourceBrowser_hidden")
      @sourceBrowser.width = sourceBrowser_width if sourceBrowser_width != nil
      @sourceBrowser.handle(self, MKUINT(ID_HIDE, SEL_COMMAND), nil) if sourceBrowser_hidden

      propFrame_width  = session.restore("propFrame_width")
      propFrame_hidden = session.restore("propFrame_hidden")
      @propFrame.width = propFrame_width if propFrame_width != nil
      @propFrame.handle(self, MKUINT(ID_HIDE, SEL_COMMAND), nil) if propFrame_hidden
      
      if main_x and main_y and main_width and main_height
        position(main_x, main_y, main_width, main_height)
        show
      else
        show(PLACEMENT_SCREEN)
      end
      
			sourceBrowser_mode	= session.restore("sourceBrowser_mode")
			@sourceBrowser.mode = sourceBrowser_mode if sourceBrowser_mode != nil
			
      #
      # post-GUI initialization
      #

      @sourceBrowser.slave  = @editBook
      @reportFrame.slave    = @editBook
      
      @editBook.add_observer(self)

			register_default_viewers      
      
      launch_home_page
			
			apply_preferences
			
			#
			# handle command-line parameters
			#
			
			if ARGV[0]
				begin
					file = FileManager.instance.open_file(ARGV[0])
					if file.kind_of?(ProjectFile)
						SessionManager.instance.open_project(file)
					else
						@editBook.launch_viewer(nil, file)
					end
				rescue
					FXMessageBox.error(self, MBOX_OK, "Parameter Error", $!)
				end
			end
    end
    
    
		def apply_preferences
			set_default_preferences # this fixes any incorrect values but leaves changed values alone
		
			session = SessionManager.instance
			
			ExecutionManager.instance.capture_stdout 	= session.restore("Preferences/Capture Stdout")
			ExecutionManager.instance.terminal_string = session.restore("Preferences/Terminal String")
			
			@editBook.each_viewer { |viewer|
				viewer.refresh
			}
		end
		
		
		def set_default_preferences(override = false)
			session = SessionManager.instance
			
			session.store("Preferences/Ruby Path", "ruby") if session.restore("Preferences/Ruby Path") == nil or session.restore("Preferences/Ruby Path") == "" or override
			
			session.store("Preferences/Auto Save", false) if session.restore("Preferences/Auto Save") == nil or override
			session.store("Preferences/Capture Stdout", false) if session.restore("Preferences/Capture Stdout") == nil or override
			
			session.store("Preferences/Line Numbers", false) if session.restore("Preferences/Line Numbers") == nil or override
			session.store("Preferences/EOL Markers", false) if session.restore("Preferences/EOL Markers") == nil or override
			session.store("Preferences/Convert Tabs", false) if session.restore("Preferences/Convert Tabs") == nil or override
			session.store("Preferences/Tab Width", 2) if session.restore("Preferences/Tab Width") == nil or override
			
			if RUBY_PLATFORM =~ /win32/
				session.store("Preferences/Web Browser", "Internet Explorer") if session.restore("Preferences/Web Browser") == nil or session.restore("Preferences/Web Browser") == "" or override
				session.store("Preferences/Terminal String", "") or override
			else
				session.store("Preferences/Web Browser", "netscape") if session.restore("Preferences/Web Browser") == nil or session.restore("Preferences/Web Browser") == "" or override
				session.store("Preferences/Terminal String", "xterm -e") if session.restore("Preferences/Terminal String") == nil or session.restore("Preferences/Terminal String") == "" or override
			end
			
			session.store("Preferences/Use Custom Styles", false) if session.restore("Preferences/Use Custom Styles") == nil or override
			session.store("Preferences/Editor Styles", EditorStyles.new) if session.restore("Preferences/Editor Styles") == nil or override
		end
		
		
		def onEditPreferences(sender, message, data)
      dlg = PreferencesDialog.new(self)
			
			dlg.execute(PLACEMENT_SCREEN)
		end
		
		
    def onCloseFile(sender, message, data)
      @editBook.close_current_viewer
    end


    def onQuit(sender, message, data)
			# close all open files and projects
			FileManager.instance.each(ProjectFile) { |project| return unless project.close }
			FileManager.instance.each { |file| return unless file.close }
		
			# save session data
      session = SessionManager.instance
      
			unless iconified?
				session.store("main_x", x)
				session.store("main_y", y)
				session.store("main_width", width)
				session.store("main_height", height)
			end
      
      session.store("reportFrame_height", @reportFrame.height)
      session.store("reportFrame_hidden",  !@reportFrame.shown?)
      session.store("sourceBrowser_width", @sourceBrowser.width)
      session.store("sourceBrowser_hidden", !@sourceBrowser.shown?)
      session.store("propFrame_width", @propFrame.width)
      session.store("propFrame_hidden", !@propFrame.shown?)
			
			session.store("sourceBrowser_mode", @sourceBrowser.mode)
      
      session.save
      
      # quit the application
      getApp.handle(self, MKUINT(FXApp::ID_QUIT, SEL_COMMAND), nil)
    end
    
    
    def onOpenFile(sender, message, data)
      openDlg = FXFileDialog.new(self, "Open File")
      
      openDlg.selectMode      = SELECTFILE_EXISTING
      openDlg.patternList     = ["All Files (*)", "Ruby Files (*.rb,*.rbw)"]
      openDlg.currentPattern  = 1
      
			begin
				file = FileManager.instance.open_file(openDlg.filename) if openDlg.execute(PLACEMENT_SCREEN) != 0
			rescue
				FXMessageBox.error(FXApp.instance.mainWindow, MBOX_OK, "Open File", $!)
				return
			end
      
			if file
				SessionManager.instance.set_working_directory(file.directory)
				
				if file.kind_of?(RubyFile)
					@editBook.launch_viewer(nil, file)
				else
					FXMessageBox.error(FXApp.instance.mainWindow, MBOX_OK, "Open File", "Unsupported file type for this operation.")
					file.close
				end
			end
    end
    
    
    def onNewFile(sender, message, data)
      @editBook.launch_viewer(RubyText, nil)
    end
    
    
    def onNewProject(sender, message, data)
      newDlg = CreateProjectDialog.new(self)
      
      if newDlg.execute(PLACEMENT_SCREEN) == 1
        SessionManager.instance.open_project(newDlg.project)
      end
    end
    
    
    def onOpenProject(sender, message, data)
      # prompt the user to select a project file
      
      openDlg = FXFileDialog.new(self, "Open Project")
      
      openDlg.selectMode      = SELECTFILE_EXISTING
      openDlg.patternList     = ["All Files (*)", "Project Files (*.mpr)"]
      openDlg.currentPattern  = 1
 
      if openDlg.execute(PLACEMENT_SCREEN) != 0      
				begin
					project = FileManager.instance.open_file(openDlg.filename)
				rescue
					FXMessageBox.error(FXApp.instance.mainWindow, MBOX_OK, "Open Project", $!)
					return
				end

        unless project.kind_of?(ProjectFile)
          FXMessageBox.error(self, MBOX_OK, "Open Project", "'#{openDlg.filename}' is not a project file.")
					return
        end
      end
      
      SessionManager.instance.open_project(project) if project
    end
    
    
		def onCloseProject(sender, message, data)
			SessionManager.instance.active_project.close
		end
		
		
    def onAddFiles(sender, message, data)
			SessionManager.instance.active_project.add_files if SessionManager.instance.active_project
    end
    
		
		def onProjectSettings(sender, message, data)
			SessionManager.instance.active_project.project_settings if SessionManager.instance.active_project
		end
		
    
		def onAddCurrentFile(sender, message, data)
			begin
				SessionManager.instance.active_project.add_file(@focused_viewer.file) if 
					SessionManager.instance.active_project and @focused_viewer and @focused_viewer.file
			rescue
				FXMessageBox.error(FXApp.instance.mainWindow, MBOX_OK, "Add Current File", "Could not add file '#{@focused_viewer.file.name}'.\nThe file needs to be saved first.")
			end

			sync_capabilities

			@focused_viewer.setFocus if @focused_viewer
		end
		
		
		def onRemoveCurrentFile(sender, message, data)
      SessionManager.instance.get_associated_project(@focused_viewer.file).remove_file(@focused_viewer.file) if 
				SessionManager.instance.get_associated_project(@focused_viewer.file) and @focused_viewer and @focused_viewer.file

			sync_capabilities

			@focused_viewer.setFocus if @focused_viewer
		end
		
		
		def onSetCurrentFileAsProjectMain(sender, message, data)
      SessionManager.instance.get_associated_project(@focused_viewer.file).project_key = @focused_viewer.file if
				SessionManager.instance.get_associated_project(@focused_viewer.file) and @focused_viewer and @focused_viewer.file
			
			sync_capabilities
			
			@focused_viewer.setFocus if @focused_viewer
		end
		
		
    def onRunFile(sender, message, data)
      @focused_viewer.file.run if @focused_viewer and @focused_viewer.file
			
			@focused_viewer.setFocus if @focused_viewer
    end
		
		
		def onRunProject(sender, message, data)
			SessionManager.instance.active_project.run if SessionManager.instance.active_project
		end
    
    
    def onStopExecution(sender, message, data)
      ExecutionManager.instance.stop_execution
    end
    
    
    def onUndo(sender, message, data)
			@focused_viewer.invoke(CMD_UNDO, data)
    end
    
    
    def onRedo(sender, message, data)
			@focused_viewer.invoke(CMD_REDO, data)
    end
    
    
    def onCut(sender, message, data)
			@focused_viewer.invoke(CMD_CUT, data)
    end
    
    
    def onCopy(sender, message, data)
			@focused_viewer.invoke(CMD_COPY, data)
    end
    
    
    def onPaste(sender, message, data)
			@focused_viewer.invoke(CMD_PASTE, data)
    end
    
    
    def onFind(sender, message, data)
			@focused_viewer.invoke(CMD_FIND)
    end
		
		
    def onReplace(sender, message, data)
      @focused_viewer.invoke(CMD_REPLACE) if @focused_viewer
    end
		
		
		def onQuickFind(sender, message, data)
			return if data == "" or 
								!@gui["toolbar.Quick Find"].children[0].hasFocus? # hack...combobox has no TEXTFIELD_ENTER_ONLY option
			
			@focused_viewer.invoke(CMD_FIND_NEXT, data) if @focused_viewer and @focused_viewer.capable?(GUI_QUICK_FIND)
			
			(@gui["toolbar.Quick Find"].numItems - 1).step(0, -1) { |index| 
				@gui["toolbar.Quick Find"].removeItem(index) if @gui["toolbar.Quick Find"].retrieveItem(index) == data
			}
			
			@gui["toolbar.Quick Find"].prependItem(data)
			@gui["toolbar.Quick Find"].currentItem = 0
			
			@gui["toolbar.Quick Find"].first.setFocus
			@gui["toolbar.Quick Find"].children[0].handle(nil, MKUINT(FXTextField::ID_CURSOR_END, SEL_COMMAND), nil) # the cursor gets screwed up when removing/adding items
		end
		
		
    def onFindNext(sender, message, data)
			@focused_viewer.invoke(CMD_FIND_NEXT) if @focused_viewer
    end
    
		
		def onFindPrevious(sender, message, data)
			@focused_viewer.invoke(CMD_FIND_PREVIOUS) if @focused_viewer
    end
		
		
		def onGotoLine(sender, message, data)
			@focused_viewer.invoke(CMD_GOTO_LINE) if @focused_viewer
		end
		
		
    def onDelete(sender, message, data)
			@focused_viewer.invoke(CMD_DELETE) if @focused_viewer
    end
  
    
    def onSelectAll(sender, message, data)
			@focused_viewer.invoke(CMD_SELECT_ALL) if @focused_viewer
    end
    
    
    def onBlockComment(sender, message, data)
			@focused_viewer.invoke(CMD_BLOCK_COMMENT) if @focused_viewer
    end
		
		
    def onSave(sender, message, data)
			@focused_viewer.file.save if @focused_viewer and @focused_viewer.file
			
			@focused_viewer.setFocus if @focused_viewer
    end
    
    
    def onSaveAs(sender, message, data)
			@focused_viewer.file.save_as if @focused_viewer and @focused_viewer.file
			
			@focused_viewer.setFocus if @focused_viewer
    end
		
		
		def onSaveAll(sender, message, data)
			FileManager.instance.each { |file| file.save }
			
			@focused_viewer.setFocus if @focused_viewer
		end
		
		
		def onEditParameters(sender, message, data)
			@focused_viewer.file.edit_parameters if @focused_viewer and @focused_viewer.file
			
			@focused_viewer.setFocus if @focused_viewer
		end
    
    
		def onEditEnvironment(sender, message, data)
			@focused_viewer.file.edit_environment if @focused_viewer and @focused_viewer.file
			
			@focused_viewer.setFocus if @focused_viewer
		end
		
		
		def onRenameFile(sender, message, data)
			@focused_viewer.file.rename if @focused_viewer and @focused_viewer.file
			
			@focused_viewer.setFocus if @focused_viewer
		end
		
		
    def onViewHome(sender, message, data)
      if sender.checked?
        close_home_page
      else
        launch_home_page
      end
    end
		
		
		def onAbout(sender, message, data)
      dlg = FXDialogBox.new(self, "About Mondrian", DECOR_TITLE | DECOR_BORDER | DECOR_CLOSE, 0, 0, 0, 0)
			dlg.backColor = "White"
      
      frame = FXVerticalFrame.new(dlg, FRAME_NONE, 0, 0, 0, 0)
			frame.backColor = "White"
			
			group_box = FXGroupBox.new(frame, "Program Details", GROUPBOX_NORMAL | FRAME_LINE | LAYOUT_FILL_X)
			group_box.backColor = "White"
			
			f = FXHorizontalFrame.new(group_box, FRAME_NONE, 0, 0, 0, 0, 0, 0, 0, 0)
			f.backColor = "White"

			iconView    = FXImageView.new(f, nil, nil, 0, LAYOUT_FIX_HEIGHT | LAYOUT_FIX_WIDTH | LAYOUT_CENTER_Y)
			iconImage   = FXBMPImage.new(parent.getApp, nil, IMAGE_KEEP)
			FXFileStream.open(SessionManager.instance.program_directory + File::Separator + "images" + File::Separator + "mondrian_icon.bmp", FXStreamLoad) do |stream| 
				iconImage.loadPixels(stream)
			end
			iconView.width  = iconImage.width
			iconView.height = iconImage.height
			iconView.image  = iconImage

			f = FXVerticalFrame.new(f, FRAME_NONE, 0, 0, 0, 0, 0, 0, 0, 0)
			f.backColor = "White"

			FXLabel.new(f, SessionManager.instance.program_name_with_version, nil, LABEL_NORMAL).backColor = "White"
			FXLabel.new(f, "Email support at #{SessionManager.instance.support_email_address}.").backColor = "White"

			FXLabel.new(group_box, "Get help or information at #{SessionManager.instance.product_homepage}.").backColor = "White"
			FXLabel.new(group_box, "Copyright (c) 2004 #{SessionManager.instance.company_name}  All rights reserved.").backColor = "White"
			
			group_box = FXGroupBox.new(frame, "Supporting Software (thanks!)", GROUPBOX_NORMAL | FRAME_LINE | LAYOUT_FILL_X)
			group_box.backColor = "White"
			
			FXLabel.new(group_box, "Ruby v#{RUBY_VERSION} is copyright (c) Yukihiro Matsumoto").backColor = "White"
			FXLabel.new(group_box, "The FOX GUI toolkit v#{Fox::fxversion} is copyright (c) Jeroen van der Zijp").backColor = "White"
			FXLabel.new(group_box, "FXRuby v#{Fox::fxrubyversion} is copyright (c) J. Lyle Johnson").backColor = "White"
			FXLabel.new(group_box, "Scintilla is copyright (c) Neil Hodgson").backColor = "White"
			FXLabel.new(group_box, "FXScintilla is copyright (c) Gilles Filippini").backColor = "White"
			
			frame = FXHorizontalFrame.new(dlg, FRAME_NONE | LAYOUT_CENTER_X, 0, 0, 0, 0)
			frame.backColor = "White"
			
      button = FXButton.new(frame, "OK", nil, dlg, FXDialogBox::ID_ACCEPT, BUTTON_NORMAL | LAYOUT_FIX_WIDTH)
      button.resize(70, 0)
			button.setFocus
			
			alive = true
			
			rotate = Proc.new { 
				if alive
					iconView.image.rotate([90, 180, 270][rand(3)])
					iconView.update
					FXApp.instance.addTimeout(2000) { rotate.call }
				end
			}
			
			rotate.call
			
			dlg.execute(PLACEMENT_OWNER)
			
			alive = false
		end
    
    
    def launch_home_page
      @editBook.launch_viewer(HomeViewer)
      @gui["menu.View/Home Page"].check
    end
    
    
    def close_home_page
      @editBook.each_viewer(HomeViewer) { |home| @editBook.close_viewer(home) }
      @gui["menu.View/Home Page"].uncheck
    end
    
    
    def onViewSidebar(sender, message, data)
      case sender
        when @gui["menu.View/Source Browser"]
          sidebar = @sourceBrowser
        when @gui["menu.View/Properties Frame"]
          sidebar = @propFrame
        when @gui["menu.View/Output Frame"]
          sidebar = @reportFrame
      end
      
      return unless sidebar
      
      if sender.checked?
        sidebar.handle(self, MKUINT(ID_HIDE, SEL_COMMAND), nil)
      else
        sidebar.handle(self, MKUINT(ID_SHOW, SEL_COMMAND), nil)
      end
    end
    
    
    def onSidebarChange(sender, message, data)
      case sender
        when @sourceBrowser
          menu = @gui["menu.View/Source Browser"]
        when @propFrame
          menu = @gui["menu.View/Properties Frame"]
        when @reportFrame
          menu = @gui["menu.View/Output Frame"]
      end
      
      return unless menu

      if sender.shown?
        menu.check
      else
        menu.uncheck
      end
    end
    
    
    def update(sender, message, data)
      case sender
        when @focused_viewer
          case message
            when CAPABILITY_ADDED, CAPABILITY_REMOVED
              sync_capabilities
							
						when CAPABILITY_UPDATE
							case data[0]
								when GUI_CURSOR_POSITION
									@gui["statusbar.Line"].text 	= data[1][0].to_s
									@gui["statusbar.Column"].text = data[1][1].to_s
							end
          end
        
        when @editBook
          case message
            when VIEWER_CLOSED # kind of a hack to uncheck the View Home Page option
              @gui["menu.View/Home Page"].uncheck if data.instance_of?(HomeViewer)
							
            when FOCUS_CHANGED
              @focused_viewer.delete_observer(self) if @focused_viewer
              @focused_viewer = data
              @focused_viewer.add_observer(self) if @focused_viewer
              
              sync_capabilities # enable/disable menu items and toolbar buttons
          end
        
        when ExecutionManager.instance
          case message
            when EXECUTION_BEGIN, EXECUTION_END
              sync_capabilities
          end
        
        when FileManager.instance
          case message
            when FILE_OPENED    # catch open file requests and parse the file...it would
              parse_file(data)  # happen anyway, but this lets us throw up a wait cursor
          end
          
        when SessionManager.instance
          case message
            when PROJECT_OPENED
              # create a new project item in the 'Set Active Project' menu
              command = FXMenuCommand.new(@gui["menu.Project/Set Active Project"].getMenu, "#{data.project_name}\t\tSet the currently active project.")
              @gui["menu.Project/Set Active Project/#{data.project_name}"] = command
              command.connect(SEL_COMMAND) { SessionManager.instance.set_active_project(data) }
              command.create
              
              sync_capabilities # activate the project menu/toolbar commands
              
              @editBook.launch_viewer(nil, data)
            
            when PROJECT_REOPENED
              @editBook.launch_viewer(nil, data)
							
            when ACTIVE_PROJECT_CHANGED
              @gui["menu.Project/Set Active Project/#{data.project_name}"].uncheck if data
              @gui["menu.Project/Set Active Project/#{SessionManager.instance.active_project.project_name}"].check if SessionManager.instance.active_project
						
						when PROJECT_CLOSED
							@gui["menu.Project/Set Active Project"].getMenu.removeChild(@gui["menu.Project/Set Active Project/#{data.project_name}"])
              @gui.delete("menu.Project/Set Active Project/#{data.project_name}")

							sync_capabilities
							
						when STATUS_MESSAGE
							@statusbar.statusline.text = data
          end
      end
    end
    
    
    def parse_file(file)
      return unless file.kind_of?(RubyFile) and not file.is_new?

      @statusbar.statusline.text = "Parsing #{file.full_name}..."
      
      getApp.beginWaitCursor {
        parser = file.parser # trigger the parser to build itself
      }
      
      @statusbar.statusline.text = ""
    end
  end

end



$> = $stderr # set the default output to standard error

application = FXApp.new("Mondrian Ruby IDE", "Brillig Systems, Inc.")

# get rid of the default font boldness on linux
font 					= application.normalFont
fd 						= font.fontDesc
fd.weight			= FONTWEIGHT_REGULAR
fd.size 			= 87 unless RUBY_PLATFORM =~ /win32/
font.fontDesc	= fd

Mondrian::MainWindow.new(application)

application.create

application.tooltipTime 	= 5000
application.tooltipPause 	= 250

begin
	application.run
rescue
	if FXMessageBox.error(FXApp.instance.mainWindow, MBOX_YES_NO, "Fatal Error", "Caught unhandled exception:\n'#{$!}'\n\nPlease send bug reports to #{SessionManager.instance.support_email_address}.\n\nWould you like to try and save all files now?") == MBOX_CLICKED_YES
		FileManager.instance.each { |file| file.save }
	end
	
	raise
end